/* Program: ex4-1.c
 * ---------------------------------------------------------------------
 * Solution to exercise 1, series 4.
 *
 * This program works with a lexicon represented as a binary search
 * tree.
 * ---------------------------------------------------------------------
 * Nikolaos S. Papaspyrou (nickie@softlab.ntua.gr)
 * National Technical University of Athens
 * Department of Electrical Engineering
 * ---------------------------------------------------------------------
 * Course: Programming Techniques (2nd semester)
 * URL:    http://www.softlab.ntua.gr/~nickie/Courses/progtech/
 * ---------------------------------------------------------------------
 */


/* Header files */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/* Definition of binary tree */

#define M 32

typedef struct nodeTag {
	char word[M];
	struct nodeTag * left, * right;
} node, * tree;


/* Inserts a word in the tree */

tree insert (tree t, const char w[M])
{
   if (t == NULL) {
      t = (node *) malloc(sizeof(node));
      
      if (t == NULL) {
         fprintf(stderr, "Out of memory\n");
         exit(1);
      }
      
      strcpy(t->word, w);
      t->left = t->right = NULL;
   }
   else {
      int cmp = strcmp(w, t->word);
      
      if (cmp < 0)
         t->left = insert(t->left, w);
      else if (cmp > 0)
         t->right = insert(t->right, w);
      else
         printf("The word '%s' already exists in the lexicon\n", w);
   }
   return t;         
}


/* Looks up a word in the tree */

int lookup (tree t, const char w[M])
{
   if (t == NULL)
      return 0;
   else {
      int cmp = strcmp(w, t->word);
      
      if (cmp < 0)
         return lookup(t->left, w);
      else if (cmp > 0)
         return lookup(t->right, w);
      else
         return 1;
   }
}
   

/* Prints all words lexicon in lexicographic order */

void printSorted (tree t)
{
	if (t != NULL) {
		printSorted(t->left);
		printf("   %s\n", t->word);
		printSorted(t->right);
	}
}


/* Main program */

int main ()
{
	tree t = NULL;
	FILE * f;
	char w[M];
	
	/* Read the lexicon from the file */
	
	if ((f = fopen("words", "rt")) == NULL) {
	   fprintf(stderr, "Cannot open file 'words'\n");
	   return 1;
	}
	
	while (!feof(f))
	   if (fscanf(f, "%s\n", w) != NULL) {
         printf("Reading: %s\n", w);
	      t = insert(t, w);
	   }
	
   /* Read a word from the standard input */
   
   printf("\nGive a word: ");
   scanf("%s", w);
   
   if (lookup(t, w))
      printf("I found the word in the lexicon\n");
   else
      printf("I didn't find the word in the lexicon\n");
   
   /* Print all words in lexicographic order */
   
   printf("\nThe lexicon contains:\n");
   printSorted(t);

	return 0;
}
