/* Program: ex4-2.c
 * ---------------------------------------------------------------------
 * Solution to exercise 2, series 4.
 *
 * This program works with (balanced) binary trees containing nodes
 * with a unique key and a real number.
 * ---------------------------------------------------------------------
 * Nikolaos S. Papaspyrou (nickie@softlab.ntua.gr)
 * National Technical University of Athens
 * Department of Electrical Engineering
 * ---------------------------------------------------------------------
 * Course: Programming Techniques (2nd semester)
 * URL:    http://www.softlab.ntua.gr/~nickie/Courses/progtech/
 * ---------------------------------------------------------------------
 */


/* Header files */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/* Definition of binary tree */

typedef struct nodeTag {
	int key;
	double data;
	struct nodeTag * left, * right;
} node, * tree;


/* Count the nodes and find the maximum cost (=depth) */

void countAndMaxCostAuxiliary (tree t, int * count, int * maxCost,
                               int currDepth)
{
   if (t != NULL) {
      (*count)++;
      if (currDepth > *maxCost)
         *maxCost = currDepth;
      countAndMaxCostAuxiliary(t->left, count, maxCost, currDepth+1);
      countAndMaxCostAuxiliary(t->right, count, maxCost, currDepth+1);
   }
}

void countAndMaxCost (tree t, int * count, int * maxCost)
{
   *count = 0;
   *maxCost = 0;
   countAndMaxCostAuxiliary(t, count, maxCost, 1);
}


/* Finds the maximal value and returns the key to it, or -1 */

void findMaxAuxiliary (tree t, double * max, int * maxKey)
{
   if (t != NULL) {
      if (t->data > *max) {
         *max = t->data;
         *maxKey = t->key;
      }
      findMaxAuxiliary(t->left, max, maxKey);
      findMaxAuxiliary(t->right, max, maxKey);
   }
}

int findMax (tree t)
{
   double max;
   int maxKey;
   
   if (t == NULL)
      return -1;
   else {
      max = t->data;
      maxKey = t->key;
      findMaxAuxiliary(t, &max, &maxKey);
      return maxKey;
   }      
}


/* Checks if a tree is balanced.
   Assuming that every subtree must be balanced.
 */

int isBalancedAuxiliary (tree t)
{
   if (t == NULL)
      return 0;
   else {
      int h1, h2;
      
      if ((h1 = isBalancedAuxiliary(t->left)) < 0)
         return -1;
      else if ((h2 = isBalancedAuxiliary(t->right)) < 0)
         return -1;
      else if (abs(h1-h2) > 1)
         return -1;
      else
         return 1 + (h1 > h2 ? h1 : h2);
   }
}

int isBalanced (tree t)
{
   return isBalancedAuxiliary(t) >= 0;
}


/* Inserts a node in the tree, so that binary search is possible on data.
   Double elements are allowed and go to the right subtree, unless the
   macro NO_DUPLICATE has been defined.
 */

#define NO_DUPLICATE

tree insert (tree t, int key, double data)
{
   if (t == NULL) {
      t = (node *) malloc(sizeof(node));
      
      if (t == NULL) {
         fprintf(stderr, "Out of memory\n");
         exit(1);
      }
      
      t->key = key;
      t->data = data;
      t->left = t->right = NULL;
   }
   else if (data < t->data)
      t->left = insert(t->left, key, data);
#ifdef NO_DUPLICATE
   else if (data > t->data)
#else
   else
#endif
      t->right = insert(t->right, key, data);
   return t;         
}


/* Main program */

int main ()
{
	tree t = NULL;
	int count, maxCost, maxKey;

   /* Read the numbers */
   
	for (;;) {
	   int key;
	   double data;
	   
	   if (scanf("%d %lf", &key, &data) == 2) {
         printf("Adding key %d and data %lg\n", key, data);
	      t = insert(t, key, data);
	   }
	   else
	      break;
	}
	
   countAndMaxCost(t, &count, &maxCost);
   printf("There are %d nodes in the tree\n", count);
   printf("The maximum cost is %d\n", maxCost);
   
   maxKey = findMax(t);
   if (maxKey < 0)
      printf("The tree is empty\n");
   else
      printf("The maximal value has key %d\n", maxKey);

   if (isBalanced(t))
      printf("The tree is balanced\n");
   else
      printf("The tree is not balanced\n");
   
	return 0;
}
