/* Program: ex4-3.c
 * ---------------------------------------------------------------------
 * Solution to exercise 3, series 4.
 *
 * This program transforms a binary tree to an array.
 * ---------------------------------------------------------------------
 * Nikolaos S. Papaspyrou (nickie@softlab.ntua.gr)
 * National Technical University of Athens
 * Department of Electrical Engineering
 * ---------------------------------------------------------------------
 * Course: Programming Techniques (2nd semester)
 * URL:    http://www.softlab.ntua.gr/~nickie/Courses/progtech/
 * ---------------------------------------------------------------------
 */


/* Header files */

#include <stdio.h>
#include <stdlib.h>


/* Definition of binary tree */

typedef struct nodeTag {
	int data;
	struct nodeTag * left, * right;
} node, * tree;


/* Definition of the array */

typedef struct {
	int data, left, right;
} ArrayNode;


/* Auxiliary function.
   It transforms a tree t to an array a of up to n elements,
   starting from index i.  It returns the number of array
   elements that it used.
 */

int treeToArrayAuxil (tree t, ArrayNode a[], int n, int i)
{
	int nleft, nright;
	
	if (t == NULL)
		return 0;
	if (i >= n)
		return -1;

	a[i].data = t->data;

	nleft = treeToArrayAuxil(t->left, a, n, i+1);
	a[i].left = (nleft > 0) ? i+1 : 0;
	if (nleft < 0)
		return nleft;

	nright = treeToArrayAuxil(t->right, a, n, i+1+nleft);
	a[i].right = (nright > 0) ? i+1+nleft : 0;
	if (nright < 0)
		return nright;
	
	return 1+nleft+nright;
}	


/* Transform the tree t to an array a of up to n elements */

int treeToArray (tree t, ArrayNode a[], int n)
{
	return treeToArrayAuxil(t, a, n, 0);
}


/* Auxiliary functions to create and print trees */

const tree empty = NULL;

tree make (int x, tree left, tree right)
{
	tree n;
	
	if ((n = (tree) malloc(sizeof(node))) == NULL) {
		fprintf(stderr, "Out of memory\n");
		exit(1);
	}
	n->data = x;
	n->left = left;
	n->right = right;
	return n;
}

void printTree (tree t)
{
	if (t != NULL) {
		printf("%d(", t->data);
		printTree(t->left);
		printf("|");
		printTree(t->right);
		printf(")");
	}
}


/* Auxiliary function to print the array as it is */

void printArray (int n, ArrayNode a[])
{
	int i;
	
	for (i=0; i<n; i++)
		printf("data=%d, left=%d, right=%d\n", a[i].data, a[i].left, a[i].right);
}


/* Auxiliary function to print the array as a tree */

void printArrayAsTree(ArrayNode a[], int n)
{
	printf("%d(", a[n].data);
	if (a[n].left > 0)
		printArrayAsTree(a, a[n].left);
	printf("|");
	if (a[n].right > 0)
		printArrayAsTree(a, a[n].right);
	printf(")");
}


/* Main program */

#define MAX 16

int main ()
{
	tree t =
		make(1,
			make(2,
				make(3, empty, empty),
				make(4,
					make(5, empty, empty),
					empty)),
			make(6,
				make(7,
					empty,
					make(8, empty, empty)),
				make(9, empty, empty)));
	ArrayNode a[MAX];
	int n;
	
	printf("Here is the tree:\n");
	printTree(t);
	printf("\n");

	printf("\nHere is the array:\n");	
	n = treeToArray(t, a, MAX);
	
	printArray(n, a);
	
	if (n > 0) {
   	printf("\nHere is the reconstructed tree:\n");	
		printArrayAsTree(a, 0);
   	printf("\n");
   }
	else
		printf("Array of %d elements is not enough for this tree\n", MAX);

	return 0;
}
