/* Program: ex4-5.c
 * ---------------------------------------------------------------------
 * Solution to exercise 5, series 4.
 *
 * This program checks whether an undirected graph is connected.
 * ---------------------------------------------------------------------
 * Nikolaos S. Papaspyrou (nickie@softlab.ntua.gr)
 * National Technical University of Athens
 * Department of Electrical Engineering
 * ---------------------------------------------------------------------
 * Course: Programming Techniques (2nd semester)
 * URL:    http://www.softlab.ntua.gr/~nickie/Courses/progtech/
 * ---------------------------------------------------------------------
 */


/* Header files */

#include <stdio.h>


/* Definition of a graph with a maximum of N vertices.
   Using an adjacency matrix representation:
   g[u][v] != 0 iff there is an edge (u, v).
 */

#define N 20

typedef unsigned char graph[N][N];


/* Check whether an undirected graph is connected.

   This is not really hard, just traverse it and check if all
   nodes have been visited.

      n       : the number of nodes in the graph
      g       : the graph's adjacency matrix
      u       : the node that we are visiting
      visited : an array where we store whether a node has been visited
      
 */

void isConnectedAuxiliary (unsigned char n, graph g,
                           unsigned char u, unsigned char visited[])
{
   unsigned char v;

   /* Visit node u */

   visited[u] = 1;

   /* For all outgoing edges (u, v) */

   for (v=0; v<n; v++)
      if (g[u][v]) {

         /* If v has not been visited, go there first */

         if (!visited[v])
            isConnectedAuxiliary(n, g, v, visited);
      }
}


int isConnected (unsigned char n, graph g)
{
   unsigned char visited[N];
   unsigned char u;

   /* Initially, all nodes are not visited */

   for (u=0; u<n; u++)
      visited[u] = 0;

   /* Starting from the first node, traverse the graph */

   isConnectedAuxiliary(n, g, 0, visited);
   
   /* If any other node has not been visited, the graph is not connected */
   
   for (u=1; u<n; u++)
      if (!visited[u])
         return 0;

   /* Otherwise, it is connected */
   
   return 1;
}


/* Main program */

int main ()
{
   graph g1 = { /* A  B  C  D  E  F  G  H  I  J */
        /* A */  { 0, 1, 1, 0, 0, 0, 0, 0, 0, 0 },
        /* B */  { 1, 0, 1, 0, 0, 0, 1, 0, 0, 0 },
        /* C */  { 1, 1, 0, 1, 1, 1, 0, 0, 0, 0 },
        /* D */  { 0, 0, 1, 0, 1, 0, 0, 0, 0, 0 },
        /* E */  { 0, 0, 1, 1, 0, 1, 0, 0, 0, 0 },
        /* F */  { 0, 0, 1, 0, 1, 0, 1, 0, 0, 0 },
        /* G */  { 0, 1, 0, 0, 0, 1, 0, 0, 0, 1 },
        /* H */  { 0, 0, 0, 0, 0, 0, 0, 0, 1, 0 },
        /* I */  { 0, 0, 0, 0, 0, 0, 0, 1, 0, 1 },
        /* J */  { 0, 0, 0, 0, 0, 0, 1, 0, 1, 0 }
   };
   graph g2 = { /* A  B  C  D  E  F  G  H  I  J */
        /* A */  { 0, 1, 1, 0, 0, 0, 0, 0, 0, 0 },
        /* B */  { 1, 0, 1, 0, 0, 0, 1, 0, 0, 0 },
        /* C */  { 1, 1, 0, 1, 1, 1, 0, 0, 0, 0 },
        /* D */  { 0, 0, 1, 0, 1, 0, 0, 0, 0, 0 },
        /* E */  { 0, 0, 1, 1, 0, 1, 0, 0, 0, 0 },
        /* F */  { 0, 0, 1, 0, 1, 0, 1, 0, 0, 0 },
        /* G */  { 0, 1, 0, 0, 0, 1, 0, 0, 0, 0 },
        /* H */  { 0, 0, 0, 0, 0, 0, 0, 0, 1, 1 },
        /* I */  { 0, 0, 0, 0, 0, 0, 0, 1, 0, 1 },
        /* J */  { 0, 0, 0, 0, 0, 0, 0, 1, 1, 0 }
   };

   if (isConnected(10, g1))
      printf("The graph G1 is connected\n");
   else
      printf("The graph G1 is not connected\n");

   if (isConnected(10, g2))
      printf("The graph G2 is connected\n");
   else
      printf("The graph G2 is not connected\n");

   return 0;
}
