/* Program: ex4-6.c
 * ---------------------------------------------------------------------
 * Solution to exercise 6, series 4.
 *
 * This program finds all cut vertices in a connected undirected graph.
 * ---------------------------------------------------------------------
 * Nikolaos S. Papaspyrou (nickie@softlab.ntua.gr)
 * National Technical University of Athens
 * Department of Electrical Engineering
 * ---------------------------------------------------------------------
 * Course: Programming Techniques (2nd semester)
 * URL:    http://www.softlab.ntua.gr/~nickie/Courses/progtech/
 * ---------------------------------------------------------------------
 */


/* Header files */

#include <stdio.h>


/* Definition of a graph with a maximum of N vertices.
   Using an adjacency matrix representation:
   g[u][v] != 0 iff there is an edge (u, v).
 */

#define N 20

typedef unsigned char graph[N][N];


/* Check whether an undirected graph is connected.

   This is not really hard, just traverse it and check if all
   nodes have been visited.

      n       : the number of nodes in the graph
      g       : the graph's adjacency matrix
      u       : the node that we are visiting
      visited : an array where we store whether a node has been visited
      
 */

void isConnectedAuxiliary (unsigned char n, graph g,
                           unsigned char u, unsigned char visited[])
{
   unsigned char v;

   /* Visit node u */

   visited[u] = 1;

   /* For all outgoing edges (u, v) */

   for (v=0; v<n; v++)
      if (g[u][v]) {

         /* If v has not been visited, go there first */

         if (!visited[v])
            isConnectedAuxiliary(n, g, v, visited);
      }
}


int isConnected (unsigned char n, graph g)
{
   unsigned char visited[N];
   unsigned char u;

   /* Initially, all nodes are not visited */

   for (u=0; u<n; u++)
      visited[u] = 0;

   /* Starting from the first node, traverse the graph */

   isConnectedAuxiliary(n, g, 0, visited);
   
   /* If any other node has not been visited, the graph is not connected */
   
   for (u=1; u<n; u++)
      if (!visited[u])
         return 0;

   /* Otherwise, it is connected */
   
   return 1;
}


/* Check whether a vertex u is a cut vertex */

int cutVertex (unsigned char n, graph g, unsigned char u)
{
   /* We can erase all the adjacent edges of u from g, but to really
      erase u itself we must create a new graph g' and copy all other
      vertices and edges to g'.  This is expensive.  Let's try something
      different that will allow us to keep all vertices in the graph g
      and compute the right result. */      
      
   unsigned char save[N];
   unsigned char v;
   int result;

   /* Erase all edges adjacent to u, but save them so that the graph can
      be restored later on */

   for (v=0; v<n; v++) {
      save[v] = g[u][v];
      g[u][v] = g[v][u] = 0;
   }
   
   /* Add one edge from u to just one other random vertex, so that the
      graph has a chance of being connected */

   v = (u+1) % n;
   g[u][v] = g[v][u] = 1;
   
   /* See if the resulting graph is connected */
   
   result = !isConnected(n, g);
   
   /* Restore the graph */
   
   for (v=0; v<n; v++)
      g[u][v] = g[v][u] = save[v];

   return result;
}


/* Main program */

int main ()
{
   graph g = { /* A  B  C  D  E  F  G  H  I  J */
       /* A */  { 0, 1, 1, 0, 0, 0, 0, 0, 0, 0 },
       /* B */  { 1, 0, 1, 0, 0, 0, 1, 0, 0, 0 },
       /* C */  { 1, 1, 0, 1, 1, 1, 0, 0, 0, 0 },
       /* D */  { 0, 0, 1, 0, 1, 0, 0, 0, 0, 0 },
       /* E */  { 0, 0, 1, 1, 0, 0, 0, 0, 0, 0 },
       /* F */  { 0, 0, 1, 0, 0, 0, 1, 0, 0, 0 },
       /* G */  { 0, 1, 0, 0, 0, 1, 0, 1, 0, 1 },
       /* H */  { 0, 0, 0, 0, 0, 0, 1, 0, 1, 0 },
       /* I */  { 0, 0, 0, 0, 0, 0, 0, 1, 0, 1 },
       /* J */  { 0, 0, 0, 0, 0, 0, 1, 0, 1, 0 }
   };
   unsigned char u;

   if (isConnected(10, g))
      for (u=0; u<10; u++)
         if (cutVertex(10, g, u))
            printf("Vertex %c is a cut vertex\n", 'A' + u);
         else
            printf("Vertex %c is not a cut vertex\n", 'A' + u);
   else
      printf("The graph G is not connected\n");
      
   return 0;
}
